@@ -22,6 +22,7 @@ use crate::prelude::StoresIter;
 use crate::query::StateQueryResult;
 use crate::query::StmtIter;
 use crate::query::Store;
+use holochain_zome_types::ScheduledFn;
 
 /// The "scratch" is an in-memory space to stage Headers to be committed at the
 /// end of the CallZome workflow.
@@ -36,6 +37,7 @@ pub struct Scratch {
     headers: Vec<SignedHeaderHashed>,
     entries: HashMap<EntryHash, Arc<Entry>>,
     chain_top_ordering: ChainTopOrdering,
+    scheduled_fns: Vec<ScheduledFn>,
 }
 
 #[derive(Debug, Clone)]
@@ -54,6 +56,14 @@ impl Scratch {
         }
     }
 
+    pub fn scheduled_fns(&self) -> &[ScheduledFn] {
+        &self.scheduled_fns
+    }
+
+    pub fn add_scheduled_fn(&mut self, scheduled_fn: ScheduledFn) {
+        self.scheduled_fns.push(scheduled_fn)
+    }
+
     pub fn chain_top_ordering(&self) -> ChainTopOrdering {
         self.chain_top_ordering
     }
@@ -89,7 +99,7 @@ impl Scratch {
     }
 
     pub fn is_empty(&self) -> bool {
-        self.headers.is_empty()
+        self.headers.is_empty() && self.scheduled_fns.is_empty()
     }
 
     pub fn headers(&self) -> impl Iterator<Item = &SignedHeaderHashed> {
@@ -148,6 +158,10 @@ impl Scratch {
         Ok(r)
     }
 
+    pub fn drain_scheduled_fns(&mut self) -> impl Iterator<Item = ScheduledFn> + '_ {
+        self.scheduled_fns.drain(..)
+    }
+
     /// Drain out all the headers.
     pub fn drain_headers(&mut self) -> impl Iterator<Item = SignedHeaderHashed> + '_ {
         self.headers.drain(..)
